CooOccurrenceClass * CooOccurranceFeatureComputation(short * pTumorDataofDWT,short * pNorData,int NormalizationMax,int nHeight_H,int nWidth_H)
{
	/*
	This  C++ source code represents how to calculate the key feature: "Contrast of Co-occurrence" and "Maximum-Probability of Co-occurrence".
	Four images were obtained from one original image by wavelet transformation (LL, LH, HL, HH), and there are four directions to calculate the Gray-level Co-occurrence Matrix (glcm). 
	pTumorDataofDWT presents the image after wavelet transformation. NormalizationMax represents the maximum value used for image normalization. nHeight_H and nWidth_H denote the height and width of the image.
	*/
	CooOccurrenceClass * CooOccurrenceFeature = new CooOccurrenceClass [4];
	memset(CooOccurrenceFeature,0,sizeof(CooOccurrenceClass)*4);
	int CooCount = 4;
	int NormalizationLimit = NormalizationMax;
	int NormalizationSize = (NormalizationLimit+1)*(NormalizationLimit+1);
	int Height = nHeight_H;
	int Width = nWidth_H;
	short * pData = new short[Height*Width];
	memset(pData,0,sizeof(short)*Height*Width);
	memcpy(pData,pNorData,sizeof(short)*Height*Width);

	short * pTumorData = new short[Height*Width];
	memset(pTumorData,0,sizeof(short)*Height*Width);
	memcpy(pTumorData,pTumorDataofDWT,sizeof(short)*Height*Width);

	short ** CooMatrixinDWT = new short *[4];
	for(int s = 0;s < CooCount;s++)
	{
		CooMatrixinDWT[s] = new short[NormalizationSize];
		memset(CooMatrixinDWT[s],0,sizeof(short)*NormalizationSize);//Matrix of Co-occurrence
	}
	for(int i = 0;i < Height;i++)
	{
		for(int j = 0;j < Width;j++)
		{
			if(pTumorData[i*Width+j] != 0)
			{
				int temp = pData[i*Width+j];
				int temp0 = pData[i*Width+j+4];
				CooMatrixinDWT[0][temp*(NormalizationLimit+1)+temp0]++;
				int temp45 = pData[(i-4)*Width+j+4];
				CooMatrixinDWT[1][temp*(NormalizationLimit+1)+temp45]++;
				int temp90 = pData[(i-4)*Width+j];
				CooMatrixinDWT[2][temp*(NormalizationLimit+1)+temp90]++;
				int temp135 = pData[(i-4)*Width+j-4];
				CooMatrixinDWT[3][temp*(NormalizationLimit+1)+temp135]++;
			}
		}
	}
	for(int i = 0;i < 4;i++)
	{
		double Contrast = 0.0;//Contrast of Co-occurrence
		double MaximumProbability = 0.0;//Maximum-Probability of Co-occurrence 

		short * px = new short [NormalizationLimit+1];
		memset(px,0,sizeof(short)*(NormalizationLimit+1));
		short * py = new short [NormalizationLimit+1];
		memset(py,0,sizeof(short)*(NormalizationLimit+1));
		double miux = 0.0;
		double miuy = 0.0;
		double sigmax = 0.0;
		double sigmay = 0.0;
		double H = 0;
		double HXY1 = 0;
		double HXY2 = 0;

		for(int j = 1;j <= NormalizationLimit;j++)
		{
			for(int k = 1;k <= NormalizationLimit;k++)
			{
				px[j] = px[j] + CooMatrixinDWT[i][j*(NormalizationLimit+1)+k];
				py[j] = py[j] + CooMatrixinDWT[i][k*(NormalizationLimit+1)+j];
			}
			miux = miux + px[j];
			miuy = miuy + py[j];
		}
		miux = miux/NormalizationLimit;
		miuy = miuy/NormalizationLimit;
		for(int j = 1;j <= NormalizationLimit;j++)
		{
			sigmax = sigmax + (px[j]-miux)*(px[j]-miux);
			sigmay = sigmay + (py[j]-miuy)*(py[j]-miuy);
		}
		sigmax = sigmax/NormalizationLimit;
		sigmay = sigmay/NormalizationLimit;
		for(int j = 1;j <= NormalizationLimit;j++)
		{
			for(int k = 1; k <= NormalizationLimit;k++)
			{
				double temp = CooMatrixinDWT[i][j*(NormalizationLimit+1)+k];

				Contrast = Contrast + temp*(j-k)*(j-k);

				if(temp > MaximumProbability)
					MaximumProbability = temp;
			}
		}
		CooOccurrenceFeature[i].Contrast = Contrast;
		CooOccurrenceFeature[i].MaximumProbability = MaximumProbability;

	}
	return CooOccurrenceFeature;
}