GaborFeatureClass *  GaborWaveletTexture::GaborWaveletTextureComputation(ccomplex ** Image,int P,int Q,short **pVolumeTumorData,int is, int nSliceHeight,int nSliceWidth)
{
	/*
	This C++ source code represents how to calculate the GaborWavelet feature.
	A real tumor image is divided into real and imaginary parts by Gabor Filter, Gabor magnitude texture representation (GMTR) and Gabor phase-based texture representation (GPTR) are captured using the convolution
	between multi-scale and multi-directional Gabor wavelet function.
	Image presents the image set after GaborWavelet transformation.
	P presents the number of scales for GaborWavelet feature calculation.
	Q presents the number of directions for GaborWavelet feature calculation.
	*/
	GaborFeatureClass * GaborWaveletSet = new GaborFeatureClass[P*Q];
	memset(GaborWaveletSet,0,sizeof(GaborWaveletFeatureClass)*P*Q);

	int PQ = P*Q;
	int Height = nSliceHeight;
	int Width = nSliceWidth;
	int SliceNumber = is;
	short * pData = new short [Height*Width];
	memcpy(pData,pVolumeTumorData[SliceNumber],sizeof(short)*Height*Width);
	double ** GMTR = new double * [PQ];
	double ** GPTR1 = new double * [PQ];
	for(int i = 0; i < PQ;i++)
	{
		GMTR[i] = new double [Width*Height];
		memset(GMTR[i],0,sizeof(double)*Width*Height);
		GPTR1[i] = new double [Width*Height];
		memset(GPTR1[i],0,sizeof(double)*Width*Height);
	}
	for(int i = 0;i < PQ;i++)
	{
		for(int j = 0; j < Width*Height;j++)
		{
			if(pData[j] == 4096)
			{
				GMTR[i][j] = sqrt(Image[i][j].Real *Image[i][j].Real + Image[i][j].Imaginary *Image[i][j].Imaginary);//Gabor magnitude texture representation (GMTR)
				GPTR1[i][j] = atan2(Image[i][j].Imaginary,Image[i][j].Real);//Gabor phase-based texture representation (GPTR) 
			}
		}
	}
	for(int i = 0; i < PQ;i++)
	{
		int intensitysum = 0;
		int pixelnumber = 0;
		int variance = 0;
		for(int j = 0;j < Height;j++)
		{
			for(int k = 0;k < Width;k++)
			{
				if(pData[j*Width+k] == 4096)
				{
					intensitysum = GMTR[i][j*Width+k] + intensitysum;
					pixelnumber++;
				}
			}
		}
		double mean = (double)intensitysum/pixelnumber;
		for(int j = 0;j < Height;j++)
		{
			for(int k = 0;k < Width;k++)
			{
				if(pData[j*Width+k] == 4096)
				{
					variance = variance + (GMTR[i][j*Width+k]-mean)*(GMTR[i][j*Width+k]-mean);
				}
			}
		}
		variance = variance/pixelnumber;
		GaborWaveletSet[i].GMTRvariance = variance;
	}

	for(int i = 0;i < PQ;i++)
	{
		short hist[256];
		memset(hist,0,sizeof(short)*256);
		int sum = 0;
		double Entropy = 0.0;
		double max = 0.0;
		double min = 4000.0;

		for(int j = 0; j < Height;j++)
		{
			for(int k = 0;k < Width;k++)
			{
				if(pData[j*Width+k] == 4096)
				{
					if(GPTR1[i][j*Width+k] < min)
						min = GPTR1[i][j*Width+k];
					if(GPTR1[i][j*Width+k] > max)
						max = GPTR1[i][j*Width+k];
				}
			}
		}


		for(int j = 0;j < Height;j++)
		{
			for(int k = 0;k < Width;k++)
			{
				if(pData[j*Width+k] == 4096)
				{
					int temp = (GPTR1[i][j*Width+k]-min)/(max-min)*255;
					hist[temp]++;
					sum++;
				}
			}
		}
		for(int ii = 0;ii < 256;ii++)
		{
			if(hist[ii] != 0)
			{
				double temp = (double)hist[ii]/sum;
				Entropy = Entropy + log(temp)*temp;
			}
		}
		GaborWaveletSet[i].GPTRentropy = Entropy;
	}
	return GaborWaveletSet;
}

 